# -*- coding: iso8859_2 -*-

#-------------------------------------------------------------------------------
# Name:        fs_global_sklepy_internetowe.py
# Purpose:     zbir funkcji dotyczcych wsppracy ze sklepami internetowymi
#
# Author:      Micha Raburski
# Created:     2011/03/18
# RCS-ID:      $Id: fs_global_sklepy_internetowe.py $
# Copyright:   (c) 2011 RecordSystem
# Licence:     GNU GPL
#-------------------------------------------------------------------------------

# 2021-08-18 Wojciech Skowron   - Klasa sklep_internetowy
# 2021-11-15 Aleksander Bargie - Brak oddziau na zamwieniach importowanych z WooCommerce (12706)
# 2021-11-24 Aleksander Bargie - Import zamwie ze sklepu  (12726)

from datetime import datetime, timedelta
import re
import rsglobal
import rssql
import traceback
from woocommerce import API

class sklep_internetowy:
    """Klasa obudowujca api sklepu internetowego

       Args:
       self_serwera (str): przekazanie self z fsa uywajcego klasy
       sklep_id (str): id sklepu z tabeli rs_sklepy_internetowe zawierajcej m.in. dane logowania
    """

    def __init__(self, self_serwera, sklep_id):
        self.sklep_id   = sklep_id
        self.firma_id   = self_serwera.firma_id
        self.oddzial_id = self_serwera.oddzial_id
        self.PolSQL     = self_serwera.PolSQL
        self.wcapi      = None

    def polacz(self):
        """Poczenie ze sklepem

           Returns:
           api_get.status_code (str): status_code pustego zapytania do api, 200 - OK
        """

        SEL = """SELECT typ, adres_url, api_klucz, api_sekret, api_parametr1, api_parametr2, api_parametr3,
                        magazyn_stany, magazyn_rezerwacje, wyznacznik_id
                 FROM rs_sklepy_internetowe
                 WHERE firma_id='{self.firma_id}' AND sklep_id='{self.sklep_id}';""".format(**locals())
        stat, dane_log, nagl_log, dump = self.PolSQL.Execute(SEL, 3, CzyDodawacBlad=True)
        if stat != 'OK':
            stat = u"Bd pobrania danych logowania do sklepu: "+stat
        elif len(dane_log) == 0:
            stat = u"Brak danych logowania do sklepu: "+stat
        else:
            self.dane_log = rssql.RSTableRowset2(dane_log, nagl_log)[0]

            self.adres_url = self.dane_log['adres_url']
            self.api_klucz = self.dane_log['api_klucz']
            self.api_sekret = self.dane_log['api_sekret']
            self.typ = self.dane_log['typ']
            self.magazyn_stany = self.dane_log['magazyn_stany']
            self.magazyn_rezerwacje = self.dane_log['magazyn_rezerwacje']
            self.wyznacznik_id = self.dane_log['wyznacznik_id']

            # ----------------------------------------------------------------------------------------------------------
            # WOOCOMMERCE
            # ----------------------------------------------------------------------------------------------------------

            if self.typ == 'WOO':
                from woocommerce import API
                self.wcapi = API(url=self.adres_url,
                                 consumer_key=self.api_klucz,
                                 consumer_secret=self.api_sekret,
                                 query_string_auth=True,
                                 version=self.dane_log['api_parametr1'])

                try:
                    api_get = self.wcapi.get("")
                    return api_get.status_code
                except:
                    rsglobal.ZapiszDoLogow(u"WOO prba poczenia ze sklepem +'"+self.sklep_id+u"' nie powioda si: "+traceback.format_exc())
                    stat = u"Wystpi bd podczas prby poczenia ze sklepem. Jeli sytuacja bdzie si powtarza, skontaktuj si z administratorem."

        return stat

    def sprawdz_przed_importem(self):
        """Funkcja sprawdzajca:
           - czy wszystkie artykuy ze sklepu posiadaj wypenione SKU
           - czy s zdefiniowane w rs_artykuly

           Returns:
           stat (str): OK lub tre bdu
           uwagi (list): lista uwag
        """

        uwagi = []

        # ----------------------------------------------------------------------------------------------------------
        # WOOCOMMERCE
        # ----------------------------------------------------------------------------------------------------------

        if self.typ == 'WOO':
            aktualna_strona = 0
            liczba_stron = 1

            while aktualna_strona < liczba_stron:
                aktualna_strona += 1

                api_get = self.wcapi.get("products", params={'per_page': 100, 'page': aktualna_strona})
                products = api_get.json()
                if api_get.status_code != 200:
                    return u'Bd pobrania danych artykuw ze sklepu: '+products['message'], uwagi
                else:
                    liczba_stron = int(api_get.headers['X-WP-TotalPages'])

                    for product in products:
                        if product['total_sales'] > 0:
                            if product['sku'] == '':
                                uwaga = u'Artyku '+str(product['id'])+' - "'+product['name']+u'" nie posiada wprowadzonego SKU'
                                if uwaga not in uwagi:
                                    uwagi.append(uwaga)
                            else:
                                stat, istnieje_artykul = self._sprawdz_czy_artykul_istnieje(product['sku'])
                                if stat == 'OK':
                                    if not istnieje_artykul:
                                        uwaga = u"Brak definicji artykuu '"+product['sku']+"'"
                                        if uwaga not in uwagi:
                                            uwagi.append(uwaga)

            return 'OK', uwagi

    def importuj_platnikow(self):
        """Import nowych patnikw

           Returns:
           stat (str): OK lub tre bdu
           liczba_zaimportowanych (int): liczba patnikw zaimportowanych
        """

        stat = 'NIC'
        liczba_zaimportowanych = 0
        liczba_zaktualizowanych = 0
        uwagi = []

        # ----------------------------------------------------------------------------------------------------------
        # Pobranie id kontrahentw ju zaimportowanych
        # ----------------------------------------------------------------------------------------------------------

        SEL = """SELECT nasz_symb FROM rs_kontrahenci WHERE firma_id='{self.firma_id}' AND nasz_symb != '';""".format(**locals())
        stat, rs_kontrahenci, dump, dump = self.PolSQL.Execute(SEL, 3, CzyDodawacBlad=True)
        if stat != 'OK':
            return u'Bd pobrania danych odbiorcw z rs_kontrahenci: '+stat, 0, 0, uwagi
        else:
            odbiorcy_wczesniej_zaimportowani = [k[0] for k in rs_kontrahenci]

        # ----------------------------------------------------------------------------------------------------------
        # WOOCOMMERCE
        # ----------------------------------------------------------------------------------------------------------

        if self.typ == 'WOO':
            aktualna_strona = 0
            liczba_stron = 1

            INS_PLAT = ""
            INS_ARCH = ""
            UPD = ""

            # pobranie najwyszego indeksu kontrahenta (S + 4 cyfry)
            SEL = "SELECT MAX(kontrahent_id) FROM rs_kontrahenci WHERE firma_id='"+self.firma_id+"' AND kontrahent_id ~ 'S\\\\d{4}';"
            stat, rs_indeks, dump, dump = self.PolSQL.Execute(SEL, 3, CzyDodawacBlad=True)
            if stat != 'OK':
                return u'Bd pobrania indeksw kontrahentw z rs_kontrahenci: '+stat, 0, 0, uwagi
            elif rs_indeks[0][0] != 'None':
                    max_id = int(rs_indeks[0][0][1:])
            else:
                max_id = 0

            # pobranie listy pl do zarchiwizowania w przypadku aktualizacji danych patnika
            SEL = """SELECT DISTINCT column_name FROM information_schema.columns 
                     WHERE table_name='rs_kontrahenci_archiwum_dane_adresowe' 
                     AND column_name NOT LIKE 'z_%' AND column_name not in ('firma_id', 'kontrahent_id', 'do_dnia');""".format(**locals())
            stat, rs_archiwum, dump, dump = self.PolSQL.Execute(SEL, 3, CzyDodawacBlad=True)
            if stat != 'OK':
                return u'Wystpi bd podczas pobrania listy pl do archiwizacji: '+stat, 0, 0, uwagi
            else:
                lista_kolumn_archiwum = ', '.join([kol[0] for kol in rs_archiwum])

            while aktualna_strona < liczba_stron:
                INS = ""

                aktualna_strona += 1

                api_get = self.wcapi.get("customers", params={'per_page': 100, 'page': aktualna_strona})
                customers = api_get.json()
                if api_get.status_code != 200:
                    return u'Bd pobrania danych odbiorcw'+customers['message'], 0, 0, uwagi
                else:
                    liczba_stron = int(api_get.headers['X-WP-TotalPages'])

                if len(customers) == 0:
                    return 'BRAK', 0, 0, uwagi
                else:
                    for customer in customers:
                        customer_id = customer["id"]

                        # dane patnoci
                        billing = customer["billing"]
                        nazwa = rsglobal.Dekoduj((billing["last_name"]+' '+billing["first_name"]).strip())[:50]
                        adr = re.compile("([0-9].*)([0-9])$").split(rsglobal.Dekoduj((billing["address_1"]+' '+billing["address_2"]).strip()))
                        adr_ulica = adr[0].strip()[:50]
                        if len(adr) > 1:
                            adr_nry = adr[1].strip()[:15]
                        else:
                            adr_nry = ''
                        adr_miej = rsglobal.Dekoduj(billing["city"].strip())[:50]
                        adr_kod = rsglobal.Dekoduj(billing["postcode"].strip())[:10]
                        email = rsglobal.Dekoduj(billing["email"].strip())
                        tel1 = rsglobal.Dekoduj(billing["phone"].strip())[:15]

                        if str(customer["id"]) not in odbiorcy_wczesniej_zaimportowani:
                            # ----------------------------------------------------------------------------------------------
                            # Import danych odbiorcw ze sklepu z wyczeniem ju zaimportowanych
                            # ----------------------------------------------------------------------------------------------

                            if INS == "":
                                INS = """INSERT INTO rs_kontrahenci (firma_id, kontrahent_id, platnik_id, nasz_symb, nazwa, 
                                                                     tel1, email, adr_ulica, adr_nry, adr_kod, adr_miej) VALUES """

                            max_id += 1
                            liczba_zaimportowanych += 1

                            platnik_id = 'S'+str(max_id).rjust(4,'0')

                            INS += """('{self.firma_id}', '{platnik_id}', NULL, '{customer_id}', '{nazwa}', 
                                       '{tel1}', '{email}', '{adr_ulica}', '{adr_nry}', '{adr_kod}', '{adr_miej}'),""".format(**locals())
                        # else:
                        #     # ----------------------------------------------------------------------------------------------
                        #     # Aktualizacja danych patnikw wczeniej zaimportowanych
                        #     # ----------------------------------------------------------------------------------------------
                        #
                        #     SEL = """SELECT kontrahent_id FROM rs_kontrahenci WHERE firma_id='{self.firma_id}' AND nasz_symb='{customer_id}' AND platnik_id IS NULL
                        #              AND (nazwa, tel1, email, adr_ulica, adr_nry, adr_kod, adr_miej) !=
                        #                   (TRIM('{nazwa}'), TRIM('{tel1}'), TRIM('{email}'), TRIM('{adr_ulica}'), TRIM('{adr_nry}'), TRIM('{adr_kod}'), TRIM('{adr_miej}'));""".format(**locals())
                        #     stat, rs_kontrahent, dump, dump = self.PolSQL.Execute(SEL, 3, CzyDodawacBlad=True)
                        #     if stat != 'OK':
                        #         return u'Bd porwnywania danych patnika z danymi sklepu: '+stat, 0, 0, uwagi
                        #     elif len(rs_kontrahent) > 0:
                        #         platnik_id = rs_kontrahent[0][0]
                        #         do_dnia = str(datetime.strptime(customer['date_modified'][:10], "%Y-%m-%d") - timedelta(days=1))[:10]
                        #
                        #         SELSPR = """SELECT 1 FROM rs_kontrahenci_archiwum_dane_adresowe
                        #                                      WHERE (firma_id, kontrahent_id, do_dnia) = ('{self.firma_id}', '{platnik_id}', '{do_dnia}');""".format(**locals())
                        #         stat, rs_archiwum, dump, dump = self.PolSQL.Execute(SELSPR, 3)
                        #         if stat != 'OK':
                        #             return u'Bd pobierania danych z archiwum kontrahentw: ' + stat, 0, 0, uwagi
                        #         elif len(rs_archiwum) > 0:
                        #             rsglobal.ZapiszDoLogow(u'WOO porwnanie danych patnika '+SEL)
                        #             uwaga = u'W dniu '+do_dnia+u' istnieje ju wpis w archiwum danych adresowych dla patnika '+platnik_id+u', aktualizacja danych nie zostaa wykonana'
                        #             rsglobal.ZapiszDoLogow(u'WOO '+uwaga)
                        #             if uwaga not in uwagi:
                        #                 uwagi.append(uwaga)
                        #         else:
                        #             liczba_zaktualizowanych += 1
                        #
                        #             # zapis aktualnych wartoci do archiwum
                        #             INS_ARCH += """INSERT INTO rs_kontrahenci_archiwum_dane_adresowe (firma_id, kontrahent_id, do_dnia, {lista_kolumn_archiwum})
                        #                            SELECT firma_id, kontrahent_id, '{do_dnia}', {lista_kolumn_archiwum}
                        #                            FROM rs_kontrahenci WHERE firma_id='{self.firma_id}' AND nasz_symb='{customer_id}' AND platnik_id IS NULL;""".format(**locals())
                        #
                        #             UPD += """UPDATE rs_kontrahenci SET
                        #                       nazwa = '{nazwa}',
                        #                       tel1 = '{tel1}',
                        #                       email = '{email}',
                        #                       adr_ulica = '{adr_ulica}',
                        #                       adr_nry = '{adr_nry}',
                        #                       adr_kod = '{adr_kod}',
                        #                       adr_miej = '{adr_miej}'
                        #                       WHERE firma_id='{self.firma_id}' AND nasz_symb='{customer_id}' AND platnik_id IS NULL;""".format(**locals())

                if INS != "":
                    INS_PLAT += INS.strip()[:-1]+";"

            if liczba_zaimportowanych > 0:
                rsglobal.ZapiszDoLogow('WOO dodanie nowych patnikw: '+INS_PLAT)
                stat, dump, dump, dump = self.PolSQL.Execute(INS_PLAT, 1, CzyDodawacBlad=True)
                if stat != 'OK':
                    return u'Bd importu patnikw: '+stat, 0, 0, uwagi

            # if liczba_zaktualizowanych > 0:
            #     rsglobal.ZapiszDoLogow('WOO archiwizacja danych patnikw: '+INS_ARCH)
            #     stat, dump, dump, dump = self.PolSQL.Execute(INS_ARCH, 1, CzyDodawacBlad=True)
            #     if stat != 'OK':
            #         return u'Bd zapisu danych patnikw do archiwum: '+stat, 0, 0, uwagi
            #
            #     rsglobal.ZapiszDoLogow('WOO aktualizacja patnikw: '+UPD)
            #     stat, dump, dump, dump = self.PolSQL.Execute(UPD, 1, CzyDodawacBlad=True)
            #     if stat != 'OK':
            #         return u'Bd aktualizacji patnikw: '+stat, 0, 0, uwagi

        return stat, liczba_zaimportowanych, liczba_zaktualizowanych, uwagi

    def __aktualizuj_platnika(self, billing, customer_id, do_dnia):
        """Funkcja pomocnicza do importu zamwie, odszukuje i zwraca id patnika po przekazanym w argumencie customer_id
           i ewentualnie aktualizuje jego dane adresowe, jeli te przekazane w argumencie rni si od tych w rs_kontrahenci

           Args:
           billing (dict): dane adresowe patnika zamwienia w formie zwrconej przez api
           customer_id (str): id patnika zamwienia ze sklepu
           do_dnia (str): data zamwienia, w wypadku aktualizacji danych adresowych patnika
                          do archiwum przenoszone s poprzednie z wpisan t dat do_dnia

           Returns:
           stat (str): OK lub tre bdu
           platnik_id (str): id patnika z rs_kontrahenci
           zaktualizowano (bool): czy zaktualizowano dane patnika w rs_kontrahenci
        """

        uwagi = []

        # ----------------------------------------------------------------------------------------------------------
        # WOOCOMMERCE
        # ----------------------------------------------------------------------------------------------------------

        if self.typ == 'WOO':
            INS_ARCH_PLAT = ""
            UPD_PLAT = ""

            zaktualizowano = False

            # pobranie listy pl do zarchiwizowania w przypadku aktualizacji danych patnika
            SEL = """SELECT DISTINCT column_name FROM information_schema.columns 
                     WHERE table_name='rs_kontrahenci_archiwum_dane_adresowe' 
                           AND column_name NOT LIKE 'z_%' AND column_name not in ('firma_id', 'kontrahent_id', 'do_dnia');""".format(**locals())
            stat, rs_archiwum, dump, dump = self.PolSQL.Execute(SEL, 3, CzyDodawacBlad=True)
            if stat != 'OK':
                return u'Wystpi bd podczas pobrania listy pl do archiwizacji: '+stat, None, False, uwagi
            else:
                lista_kolumn_archiwum = [kol[0] for kol in rs_archiwum]

            # dane patnoci
            nazwa = rsglobal.Dekoduj((billing["last_name"]+' '+billing["first_name"]).strip())[:50]
            adr = re.compile("([0-9].*)([0-9])$").split(rsglobal.Dekoduj((billing["address_1"]+' '+billing["address_2"]).strip()))
            adr_ulica = adr[0].strip()[:50]
            if len(adr) > 1:
                adr_nry = adr[1].strip()[:15]
            else:
                adr_nry = ''
            adr_miej = rsglobal.Dekoduj(billing["city"].strip())[:50]
            adr_kod = rsglobal.Dekoduj(billing["postcode"].strip())[:10]
            email = rsglobal.Dekoduj(billing["email"].strip())
            tel1 = rsglobal.Dekoduj(billing["phone"].strip())[:15]

            SEL = """SELECT kontrahent_id, (d.nazwa, d.adr_ulica, d.adr_nry, d.adr_kod, d.adr_miej) =
                                           (TRIM('{nazwa}'), TRIM('{adr_ulica}'), TRIM('{adr_nry}'), TRIM('{adr_kod}'), TRIM('{adr_miej}')) AS zgodne
                     FROM rs_kontrahenci k
                     JOIN rsf_dane_kontrahenta(firma_id, kontrahent_id, '{do_dnia}') d USING (firma_id, kontrahent_id)
                     WHERE k.firma_id='{self.firma_id}' AND k.nasz_symb='{customer_id}' AND k.platnik_id IS NULL
                     ORDER BY zgodne DESC, kontrahent_id;""".format(**locals())
            stat, rs_platnik, dump, dump = self.PolSQL.Execute(SEL, 3)
            if stat != 'OK':
                return u'Bd pobrania danych patnika: '+stat, None, False, uwagi
            elif len(rs_platnik) > 0:
                platnik_id = rs_platnik[0][0]
                if not rs_platnik[0][1]:  # dane patnoci w zamwieniu inne ni patnika
                    SELSPR = """SELECT 1 FROM rs_kontrahenci_archiwum_dane_adresowe 
                             WHERE (firma_id, kontrahent_id, do_dnia) = ('{self.firma_id}', '{platnik_id}', '{do_dnia}');""".format(**locals())
                    stat, rs_archiwum, dump, dump = self.PolSQL.Execute(SELSPR, 3)
                    if stat != 'OK':
                        return u'Bd pobierania danych z archiwum kontrahentw: ' + stat, None, False, uwagi
                    elif len(rs_archiwum) > 0:
                        rsglobal.ZapiszDoLogow(u'WOO porwnanie danych patnika '+SEL)
                        uwaga = u'W dniu '+do_dnia+u' istnieje ju wpis w archiwum danych adresowych dla patnika '+platnik_id+u', aktualizacja danych nie zostaa wykonana'
                        rsglobal.ZapiszDoLogow(u'WOO '+uwaga)
                        if uwaga not in uwagi:
                            uwagi.append(uwaga)
                    else:
                        INS_ARCH_PLAT += """INSERT INTO rs_kontrahenci_archiwum_dane_adresowe 
                                                        (firma_id, kontrahent_id, do_dnia, 
                                                         nazwa, adr_ulica, adr_nry, adr_kod, adr_miej)
                                            VALUES ('{self.firma_id}', '{platnik_id}', '{do_dnia}',
                                                    '{nazwa}', '{adr_ulica}', '{adr_nry}', '{adr_kod}', '{adr_miej}');""".format(**locals())
                        rsglobal.ZapiszDoLogow(u'WOO archiwizacja danych patnika zamwienia: '+INS_ARCH_PLAT)
                        stat, dump, dump, dump = self.PolSQL.Execute(INS_ARCH_PLAT, 1, CzyDodawacBlad=True)
                        if stat != 'OK':
                            return u'Bd aktualizacji danych patnika: '+stat, None, False, uwagi
                        else:
                            zaktualizowano = True
                return 'OK', platnik_id, zaktualizowano, uwagi
            else:
                # zamwienie bez tworzenia konta - trzeba doda nowego patnika

                # pobranie najwyszego indeksu kontrahenta (S + 4 cyfry)
                SEL = "SELECT MAX(kontrahent_id) FROM rs_kontrahenci WHERE firma_id='"+self.firma_id+"' AND kontrahent_id ~ 'S\\\\d{4}';"
                stat, rs_indeks, dump, dump = self.PolSQL.Execute(SEL, 3, CzyDodawacBlad=True)
                if stat != 'OK':
                    return u'Bd pobrania indeksw kontrahentw z rs_kontrahenci: '+stat, None, False, uwagi
                else:
                    if rs_indeks[0][0] != 'None':
                        max_id = int(rs_indeks[0][0][1:])
                    else:
                        max_id = 0

                platnik_id = 'S'+str(max_id+1).rjust(4, '0')

                INS = """INSERT INTO rs_kontrahenci (firma_id, kontrahent_id, platnik_id, nasz_symb, nazwa, 
                                                     tel1, email, adr_ulica, adr_nry, adr_kod, adr_miej) VALUES 
                                                     ('{self.firma_id}', '{platnik_id}', NULL, '{customer_id}', '{nazwa}', 
                                                      '{tel1}', '{email}', '{adr_ulica}', '{adr_nry}', '{adr_kod}', '{adr_miej}');""".format(**locals())
                rsglobal.ZapiszDoLogow(u'WOO dodanie nowego patnika: '+INS)
                stat, dump, dump, dump = self.PolSQL.Execute(INS, 1, CzyDodawacBlad=True)
                if stat != 'OK':
                    return u'Bd dodawania patnika: '+stat, None, False, uwagi
                else:
                    return 'OK', platnik_id, True, uwagi

    def __dodaj_nowego_odbiorce(self, shipping, customer_id, platnik_id):
        """Funkcja pomocnicza do importu zamwie. Odszukuje i zwraca id odbiorcy po przekazanym w argumencie customer_id,
           pod warunkiem e dane szgodne z tymi przekazanymi w argumencie, a jeli nie to tworzy nowego i zwraca jego id

           Args:
           shipping (dict): dane adresowe odbiorcy zamwienia w formie zwrconej przez api
           customer_id (str): id odbiorcy zamwienia ze sklepu
           platnik_id (str): id patnika powizanego z odbiorc

           Returns:
           stat (str): OK lub tre bdu
           kontrahent_id (str): id odbiorcy z rs_kontrahenci
        """

        # ----------------------------------------------------------------------------------------------------------
        # WOOCOMMERCE
        # ----------------------------------------------------------------------------------------------------------

        if self.typ == 'WOO':
            # dane dostawy (kontrahenta)
            nazwa = rsglobal.Dekoduj((shipping["last_name"]+' '+shipping["first_name"]).strip())[:50]
            adr = re.compile("([0-9].*)([0-9])$").split(rsglobal.Dekoduj((shipping["address_1"]+' '+shipping["address_2"])).strip())
            adr_ulica = adr[0].strip()[:50]
            if len(adr) > 1:
                adr_nry = adr[1].strip()[:15]
            else:
                adr_nry = ''
            adr_miej = rsglobal.Dekoduj(shipping["city"].strip())[:50]
            adr_kod = rsglobal.Dekoduj(shipping["postcode"].strip())[:10]

            SEL = """SELECT kontrahent_id, (nazwa, adr_ulica, adr_nry, adr_kod, adr_miej) =
                                           (TRIM('{nazwa}'), TRIM('{adr_ulica}'), TRIM('{adr_nry}'), TRIM('{adr_kod}'), TRIM('{adr_miej}')) AS zgodne
                     FROM rs_kontrahenci
                     WHERE firma_id='{self.firma_id}' AND nasz_symb='{customer_id}' AND platnik_id='{platnik_id}'
                     ORDER BY zgodne DESC, kontrahent_id;""".format(**locals())
            stat, rs_kontrahenci, dump, dump = self.PolSQL.Execute(SEL, 3)
            if stat != 'OK':
                return u'Bd pobrania danych kontrahenta: '+stat, None
            elif len(rs_kontrahenci) > 0 and rs_kontrahenci[0][1]:
                return 'OK', rs_kontrahenci[0][0]
            else:
                # pobranie najwyszego indeksu kontrahenta (S + 4 cyfry)
                SEL = "SELECT MAX(kontrahent_id) FROM rs_kontrahenci WHERE firma_id='" + self.firma_id + "' AND kontrahent_id ~ 'S\\\\d{4}';"
                stat, rs_indeks, dump, dump = self.PolSQL.Execute(SEL, 3, CzyDodawacBlad=True)
                if stat != 'OK':
                    return u'Bd pobrania indeksw kontrahentw z rs_kontrahenci: ' + stat, None
                else:
                    if rs_indeks[0][0] != 'None':
                        max_id = int(rs_indeks[0][0][1:])
                    else:
                        max_id = 0

                kontrahent_id = 'S' + str(max_id + 1).rjust(4, '0')

                INS_KONTR = """INSERT INTO rs_kontrahenci (firma_id, kontrahent_id, platnik_id, nasz_symb, nazwa, 
                                                           adr_ulica, adr_nry, adr_kod, adr_miej) VALUES 
                                                          ('{self.firma_id}', '{kontrahent_id}', '{platnik_id}', '{customer_id}', '{nazwa}', 
                                                           '{adr_ulica}', '{adr_nry}', '{adr_kod}', '{adr_miej}');""".format(**locals())
                rsglobal.ZapiszDoLogow(u'WOO dodanie odbiorcy zamwienia: '+INS_KONTR)
                stat, dump, dump, dump = self.PolSQL.Execute(INS_KONTR, 1, CzyDodawacBlad=True)
                if stat != 'OK':
                    return u'Bd dodawania odbiorcy: '+stat, None
                else:
                    return 'OK', kontrahent_id

    def _sprawdz_czy_artykul_istnieje(self, artykul_id):
        SEL = """SELECT 1 FROM rs_artykuly WHERE firma_id='{self.firma_id}' AND TRIM(artykul_id)=TRIM('{artykul_id}');""".format(**locals())
        stat, rs_artykul, dump, dump = self.PolSQL.Execute(SEL, 3, CzyDodawacBlad=True)
        if stat != 'OK':
            stat = u'Bd pobrania danych artykuw: '+stat
            return stat, False
        else:
            return 'OK', len(rs_artykul) > 0

    def importuj_zamowienia(self, zamowienia_od=str(datetime.now()-timedelta(days=1))[:10]):
        """Import oraz aktualizacja zamwie wraz z aktualizacj patnikw i dodawaniem odbiorcw

           Do prawidowego dziaania wymaga wczeniejszego wywoania funkcji importuj_platnikow().
           Dane patnoci w zamwieniu porwnywane s z zaimportowanymi danymi patnika w rs_kontrahenci.
           Jeli s rne, dane w rs_kontrahenci zostan zaktualizowane, a wczeniejsze przeniesione do archiwum
           rs_kontrahenci_archiwum_dane_adresowe z dat zamwienia wpisan w dat do_dnia.
           Dane dostawy w zamwieniu porwnywane s z zaimportowanymi danymi odbiorcy w rs_kontrahenci.
           Jeli s rne, tworzony jest nowy odbiorca w rs_kontrahenci.

           Args:
           zamowienia_od (str): data od ktrej importowa/aktualizowa zamwienia (domylnie wczorajsza)

           Returns:
           stat (str): OK lub tre bdu
           liczba_zaimportowanych (int): liczba zamwie zaimportowanych
           liczba_zaktualizowanych (int): liczba zamwie zaktualizowanych
        """

        stat = 'NIC'
        liczba_zaimportowanych = 0
        liczba_zaktualizowanych = 0
        uwagi = []

        # ----------------------------------------------------------------------------------------------------------
        # Pobranie id zamwie ju zaimportowanych
        # ----------------------------------------------------------------------------------------------------------

        SEL = """SELECT zamowienie_nr, kontrahent_id FROM rs_zamowienia_odbiorcy WHERE firma_id='{self.firma_id}' AND zamowienie_nr IS NOT NULL AND data>='{zamowienia_od}';""".format(**locals())
        stat, rs_zamowienia_odbiorcy, dump, dump = self.PolSQL.Execute(SEL, 3, CzyDodawacBlad=True)
        if stat != 'OK':
            return u'Bd pobrania danych zamwie z rs_zamowienia_odbiorcy: '+stat, 0, 0, uwagi
        else:
            zamowienia_wczesniej_zaimportowane = [(zamowienie[0], zamowienie[1]) for zamowienie in rs_zamowienia_odbiorcy]

        # ----------------------------------------------------------------------------------------------------------
        # Pobranie danych stawek VAT i utworzenie z nich sownika
        # ----------------------------------------------------------------------------------------------------------

        stawki = {}

        api_get = self.wcapi.get("taxes")
        taxes = api_get.json()
        if api_get.status_code != 200:
            return u'Bd pobrania danych zamwie ze sklepu: '+taxes['message'], 0, 0, uwagi
        else:
            for tax in taxes:
                try:
                    stawki[tax['class']] = int(float(tax['rate']))
                except:
                    stawki[tax['class']] = 0

        # ----------------------------------------------------------------------------------------------------------
        # WOOCOMMERCE
        # ----------------------------------------------------------------------------------------------------------

        if self.typ == 'WOO':
            aktualna_strona = 0
            liczba_stron = 1

            while aktualna_strona < liczba_stron:
                aktualna_strona += 1

                api_get = self.wcapi.get("orders", params={'per_page': 100, "after": zamowienia_od+'T00:00:01', 'page': aktualna_strona})
                orders = api_get.json()
                if api_get.status_code != 200:
                    return u'Bd pobrania danych zamwie ze sklepu: '+orders['message'], 0, 0, uwagi
                else:
                    liczba_stron = int(api_get.headers['X-WP-TotalPages'])

                if len(orders) == 0:
                    return u'BRAK', 0, 0, uwagi
                else:
                    UPD = ""
                    UPD_POZ = ""

                    for order in orders:
                        INS = ""
                        INS_POZ = ""

                        wyznacznik_id = self.wyznacznik_id
                        data = order['date_created'][:10] if order['date_created'] is not None else order['date_modified']
                        order_id = order['id']
                        customer_id = order['customer_id']
                        kwota_vat = float(order['total_tax'])
                        do_zaplaty = float(order['total'])
                        line_items = order['line_items']
                        tax_lines = order['tax_lines']
                        spos_zapl = order['payment_method_title']  #AB 24.11.21r

                        if order['customer_id'] in [1, ]:
                            pass
                        elif str(order["id"]) not in [zam[0] for zam in zamowienia_wczesniej_zaimportowane]:
                            # ----------------------------------------------------------------------------------------------
                            # Import danych zamwie ze sklepu z wyczeniem ju zaimportowanych
                            # ----------------------------------------------------------------------------------------------

                            # pobranie id patnika pasujcego do danych patnoci w zamwieniu i aktualizacja jego danych
                            wynik = self.__aktualizuj_platnika(order["billing"], customer_id, data)
                            stat = wynik[0]
                            if stat != 'OK':
                                return u"Bd importu zamwienia: "+stat, 0, 0, uwagi
                            else:
                                platnik_id = wynik[1]
                                uwagi.extend(wynik[3])

                            if order["billing"] != order["shipping"]:
                                # jeli istnieje kontrahent pasujcy do danych dostawy - pobranie jego id, jeli nie - utworzenie nowego
                                wynik = self.__dodaj_nowego_odbiorce(order["shipping"], customer_id, platnik_id)
                                stat = wynik[0]
                                if stat != 'OK':
                                    return u"Bd importu zamwienia: "+stat, 0, 0, uwagi
                                else:
                                    kontrahent_id = wynik[1]
                            else:
                                kontrahent_id = platnik_id

                            # pobranie numeru dla nowego zamwienia
                            SEL = "SELECT rsf_podaj_nowy_numer_dokumentu('"+self.firma_id+"',"
                            SEL+= "'[''"+wyznacznik_id+"'',''"+data+"'']','ZO');"
                            stat,rsf_numer,dump,dump = self.PolSQL.Execute(SEL, 3)
                            if stat != 'OK':
                                return u'Bd szukania numeru zamwienia: '+stat, 0, 0, uwagi
                            elif len(rsf_numer) > 0:
                                numer = rsf_numer[0][0]
                            else:
                                return u'Nie udao si pobra nowego numeru zamwienia', 0, 0, uwagi

                            liczba_zaimportowanych += 1
                            #AB 15.11.21r dodane oddzial_id
                            #AB 24.11.21r dodane zamawiajacy_id, zamowienie_data i sposob_zaplaty
                            INS = """INSERT INTO rs_zamowienia_odbiorcy (firma_id, wyznacznik_id, data, zamowienie_data, numer, oddzial_id,
                                                                          kontrahent_id, zamawiajacy_id,  platnik_id, waluta, kurs, 
                                                                          zamowienie_nr, kwota_vat, do_zaplaty, sposob_zaplaty) VALUES
                                                                         ('{self.firma_id}', '{wyznacznik_id}', '{data}', '{data}', {numer}, '{self.oddzial_id}',
                                                                          '{kontrahent_id}', '{kontrahent_id}', '{platnik_id}', 'PLN', 0,
                                                                          '{order_id}', {kwota_vat}, {do_zaplaty}, 
                                                                          (SELECT kod FROM rs_sposoby_zaplaty WHERE firma_id='{self.firma_id}' AND UPPER(TRIM(opis))=UPPER(TRIM('{spos_zapl}'))));""".format(**locals())
                            rsglobal.ZapiszDoLogow(u'WOO dodanie nagwka nowego zamwienia: '+INS)
                            stat, dump, dump, dump = self.PolSQL.Execute(INS, 1, CzyDodawacBlad=True)
                            if stat != 'OK':
                                return u'Bd dodawania nagwka zamwienia: ' + stat, 0, 0, uwagi

                            INS_POZ = """INSERT INTO rs_zamowienia_odbiorcy_pozycje
                                         (firma_id, wyznacznik_id, data, numer, nr_poz, nr_ppoz,
                                         artykul_id, stawka_vat, cena_netto1, 
                                         wartosc_netto, wartosc_vat, ilosc1)
                                         VALUES """

                            # ptla po danych pozycji
                            for nr_poz, line in enumerate(line_items):
                                artykul_id = line['sku']
                                if line['tax_class'] in stawki:
                                    stawka_vat = str(stawki[line['tax_class']]).rjust(2,'0')[:2] if stawki[line['tax_class']] >= 0 else 'zw'
                                else:
                                    stawka_vat = 'zw'
                                cena_netto1 = line['price']
                                wartosc_netto = float(line['total'])
                                wartosc_vat = float(line['total_tax'])
                                ilosc1 = line['quantity']

                                INS_POZ += """('{self.firma_id}', '{wyznacznik_id}', '{data}', {numer}, {nr_poz}+1, 0,
                                               '{artykul_id}', '{stawka_vat}', {cena_netto1},
                                               '{wartosc_netto}', '{wartosc_vat}', {ilosc1}),""".format(**locals())

                            INS_POZ = INS_POZ.strip()[:-1]+';'
                            rsglobal.ZapiszDoLogow(u'WOO dodanie pozycji nowego zamwienia: '+INS_POZ)
                            stat, dump, dump, dump = self.PolSQL.Execute(INS_POZ, 1, CzyDodawacBlad=True)
                            if stat != 'OK':
                                return u'Bd dodawania pozycji zamwienia: ' + stat, 0, 0, uwagi
                        else:
                            # ----------------------------------------------------------------------------------------------
                            # Aktualizacja danych zamwie ju zaimportowanych
                            # ----------------------------------------------------------------------------------------------

                            # pobranie id patnika pasujcego do danych patnoci w zamwieniu i aktualizacja jego danych
                            wynik = self.__aktualizuj_platnika(order["billing"], customer_id, data)
                            stat = wynik[0]
                            if stat != 'OK':
                                return u"Bd importu zamwienia: "+stat, 0, 0, uwagi
                            else:
                                platnik_id = wynik[1]

                            if order["billing"] != order["shipping"]:
                                # jeli istnieje kontrahent pasujcy do danych dostawy - pobranie jego id, jeli nie - utworzenie nowego
                                wynik = self.__dodaj_nowego_odbiorce(order["shipping"], customer_id, platnik_id)
                                stat = wynik[0]
                                if stat != 'OK':
                                    return u"Bd importu zamwienia: "+stat, 0, 0, uwagi
                                else:
                                    kontrahent_id = wynik[1]
                            else:
                                kontrahent_id = platnik_id

                            if kontrahent_id != [zam[1] for zam in zamowienia_wczesniej_zaimportowane if zam[0] == str(order["id"])][0]:
                                # dane kontrahenta ulegy zmianie - zmiana id kontrahenta w zamwieniu
                                UPD += """UPDATE rs_zamowienia_odbiorcy SET 
                                          kontrahent_id = '{kontrahent_id}'
                                          WHERE firma_id='{self.firma_id}' AND wyznacznik_id='{wyznacznik_id}' AND data='{data}' AND zamowienie_nr='{order_id}';""".format(**locals())

                            # ptla po danych pozycji
                            for nr_poz, line in enumerate(line_items):
                                artykul_id = line['sku']
                                if line['tax_class'] in stawki:
                                    stawka_vat = str(stawki[line['tax_class']]).rjust(2,'0')[:2] if stawki[line['tax_class']] >= 0 else 'zw'
                                else:
                                    stawka_vat = 'zw'
                                cena_netto1 = line['price']
                                wartosc_netto = float(line['total'])
                                wartosc_vat = float(line['total_tax'])
                                ilosc1 = line['quantity']

                                # sprawdzenie czy dane pozycji zgodne
                                SEL = """SELECT numer, (artykul_id, stawka_vat, cena_netto1, wartosc_netto, wartosc_vat, ilosc1) =
                                                       ('{artykul_id}', '{stawka_vat}', ROUND({cena_netto1},4), ROUND({wartosc_netto},4), ROUND({wartosc_vat},4), ROUND({ilosc1},4))
                                         FROM rs_zamowienia_odbiorcy_pozycje p
                                         JOIN rs_zamowienia_odbiorcy o USING (firma_id, wyznacznik_id, data, numer)
                                         WHERE p.firma_id='{self.firma_id}' AND p.wyznacznik_id='{wyznacznik_id}' AND p.data='{data}' AND o.zamowienie_nr='{order_id}' AND p.nr_poz={nr_poz}+1;""".format(**locals())
                                stat, rs_zamowienia_odbiorcy_pozycje, dump, dump = self.PolSQL.Execute(SEL, 3)
                                if stat != 'OK':
                                    return 'Bd pobrania pozycji zamwienia: '+stat, 0, 0, uwagi
                                elif len(rs_zamowienia_odbiorcy_pozycje) > 0:
                                    if not rs_zamowienia_odbiorcy_pozycje[0][1]:  # dane pozycji zmieniy si
                                        UPD_POZ += """UPDATE rs_zamowienia_odbiorcy_pozycje SET 
                                                      artykul_id = '{artykul_id}',
                                                      stawka_vat = '{stawka_vat}',
                                                      cena_netto1 = {cena_netto1},
                                                      wartosc_netto = {wartosc_netto},
                                                      wartosc_vat = {wartosc_vat},
                                                      ilosc1 = {ilosc1}
                                                      WHERE firma_id='{self.firma_id}' AND wyznacznik_id='{wyznacznik_id}' AND data='{data}' AND numer={rs_zamowienia_odbiorcy_pozycje[0][0]} AND nr_poz={nr_poz};""".format(**locals())
                                else:
                                    INS += """INSERT INTO rs_zamowienia_odbiorcy_pozycje 
                                                         (firma_id, wyznacznik_id, data, numer, nr_poz, nr_ppoz
                                                          artykul_id, stawka_vat, cena_netto1, 
                                                          wartosc_netto, wartosc_vat, ilosc1) VALUES 
                                                         ('{self.firma_id}', '{wyznacznik_id}', '{data}', {rs_zamowienia_odbiorcy_pozycje[0][0]}, {nr_poz}, 0,
                                                          '{artykul_id}', {stawka_vat}, {cena_netto1},
                                                          {wartosc_netto}, {wartosc_vat}, {ilosc1});""".format(**locals())

            if liczba_zaktualizowanych > 0:
                rsglobal.ZapiszDoLogow(u'WOO aktualizacja nagwkw zamwie: '+UPD)
                rsglobal.ZapiszDoLogow(u'WOO aktualizacja pozycji zamwie: '+UPD_POZ)
                stat, dump, dump, dump = self.PolSQL.Execute(UPD+UPD_POZ, 1, CzyDodawacBlad=True)
                if stat != 'OK':
                    return u'Bd aktualizacji danych zamwie: '+stat, 0, 0, uwagi

        return stat, liczba_zaimportowanych, liczba_zaktualizowanych, uwagi
