# -*- coding: iso8859_2 -*-
#-----------------------------------------------------------------------------
# Name:        conn.RSAnakondaSocket.py
# Purpose:     Obsuga socketw dedykowana dla programu Anakonda.
#
# Author:      Jan Kaliszewski (na podstawie poprzedniego kodu)
#
# Created:     2009/05/28
# RCS-ID:      $Id: conn.RSAnakondaSocket.py $
# Copyright:   (c) 2009 Record System
# Licence:     GNU GPL
#-----------------------------------------------------------------------------
# Jan Kaliszewski, 2009-06-25:
# * moliwo cigego sprawdzania poczenia (ad mantis: 1904)
# Jan Kaliszewski, 2009-09-01:
# * dodanie keepalive (mantis: 2474)
# Jan Kaliszewski, 2011-05-22:
# * wysyanie plikw, w tym rwnie z kompresj (wyslij_plik, Wysylacz)


import collections
import gzip
import errno
import os.path as osp
import socket
import sys
import time
import traceback
import thread

import rsglobal

from OpenSSL import SSL


if not hasattr(socket, 'SHUT_RDWR'):
    socket.SHUT_RDWR = 2

DOMYSL_KOMPRESJA = rsglobal.PodajZKonf(rsglobal.PLIK_KONFIGURACJI, ('kompresja',))[0]
if not DOMYSL_KOMPRESJA or DOMYSL_KOMPRESJA.lower() == 'auto':
    DOMYSL_KOMPRESJA = None
else:
    DOMYSL_KOMPRESJA = int(DOMYSL_KOMPRESJA)


class KillableLock(object):
    "Lock, ktry nie robi problemw przy killowaniu."

    def __init__(self):
        self.lock = thread.allocate_lock()

    def acquire(self):
        while not self.lock.acquire(0):
            # (to kosztuje nas pewne marnotrastwo czasu,
            # ktre jednak przy obecnym zastosowaniu tej
            # klasy nie ma istotnego znaczenia)
            time.sleep(0.3)

    def release(self):
        return self.lock.release()


class Wysylacz(object):
    "Pseudoplik pomocny przy kompresowaniu+wysyaniu plikw."

    # uywa osobnego wtku, dziki czemu jednoczenie
    # kompresuje i wysya przez sie, to co ju skompresowano

    INTERWAL = 0.01  # (nie chcemy lockw ze wzgldu na ew. problemy z killowaniem)
    MAX_DLUG_BUF = 2048 * 1024  # bajtw

    def __init__(self, asocket):
        self.otwarty = False
        self._asocket = asocket
        self._watek_wysylajacy = None
        self._blad = None
        self._wrzucono = 0
        self._wyslano = 0
        self._buf = collections.deque()

    def _wysylaj_w_watku(self):
        dlug_info_o_dlug = rsglobal.MAX_DLUGOSC_INFO_O_DLUGOSCI
        sendall = self._asocket.socket.sendall
        try:
            try:
                while True:
                    try:
                        s = self._buf.pop()
                    except IndexError:
                        time.sleep(self.INTERWAL)
                    else:
                        if s is not None:
                            # write
                            sendall('%*d' % (dlug_info_o_dlug, len(s)))
                            sendall(s)
                            self._wyslano += len(s)
                        else:
                            # close
                            sendall('%*d' % (dlug_info_o_dlug, 0))
                            break
            except:
                self._blad = sys.exc_info()[:2]
        finally:
            self.otwarty = False

    def _blad_z_watku(self):
        if self._blad:
            blad = self._blad
            self._blad = None
            raise blad[0], blad[1]

    def open(self):
        if self.otwarty:
            raise ValueError('%r ju otwarty' % self)
        self.otwarty = True
        self._watek_wysylajacy = thread.start_new_thread(self._wysylaj_w_watku, ())
        return self

    def write(self, s):
        if not self.otwarty:
            raise ValueError('%r nie jest otwarty' % self)
        if s:
            while self.otwarty and (self._wrzucono - self._wyslano > self.MAX_DLUG_BUF):
                time.sleep(self.INTERWAL)
            self._blad_z_watku()
            self._buf.appendleft(s)
            self._wrzucono += len(s)

    def flush(self):
        if not self.otwarty:
            raise ValueError('%r nie jest otwarty' % self)
        self._blad_z_watku()

    def close(self):
        try:
            self._blad_z_watku()
            self._buf.appendleft(None)
            while self.otwarty:
                time.sleep(self.INTERWAL)
            self._blad_z_watku()
        finally:
            self.otwarty = False
            self._blad = None
            self._wrzucono = 0
            self._wyslano = 0
            self._buf.clear()


class AnakondaSocket(object):
    "Klasa socketu dedykowana dla programu Anakonda."

    def __init__(self, sock_family=None, sock_type=None, socket_obj=None, lokal=None):
        "Inicjujemy albo ustawieniami family/type albo gotowym socketem."

        # (atrybut 'socket' jest czci API!)
        if socket_obj:
            self.socket = socket_obj
        else:
            self.socket = socket.socket(sock_family, sock_type)

        self.ustaw_keepalive()
        self.warunek_buforowania = True
        try:
            self.lokal = bool(rsglobal.RE_LOKAL_ADR.match(self.socket.getpeername()[0]))
        except (socket.error, EnvironmentError):
            self.lokal = None
        self._ustaw_reszte()


    # dla wygody osobno (bo rwnie dla klasy potomnej z innym konstruktorem):
    def _ustaw_reszte(self):
        self._blokada_spr = KillableLock()
        self._juz_odebrane = []
        self._wysylacz = Wysylacz(self)
        # (atrybut 'czy_sprawdzac' jest czci API!)
        self.czy_sprawdzac = False


    # Specjalna metoda wywoywana w razie nieznalezienia atrybutu.
    # Tu: zapewnia symulacj dziedziczenia z socket.socket
    def __getattr__(self, name):
        return getattr(self.socket, name)


    def _pobierz_kawalek(self, max_dlugosc):
        pobrane = self.socket.recv(max_dlugosc)
        if not pobrane:  # pusty napis oznacza bd
            raise socket.error
        return pobrane


    def odbierz_raz(self, calkowita_dlugosc):
        "Odbieramy napis o podanej dugoci."

        self._blokada_spr.acquire()
        try:
            # (korzystamy z faktu, e kady z elementw self._juz_odebrane
            # dodanych przy ew. sprawdzaniu poczenia to jednoznakowy napis)

            do_pobrania = calkowita_dlugosc - len(self._juz_odebrane)

            if do_pobrania >= 0:
                if rsglobal.SOCK_BUF_OPTIMUM and self.warunek_buforowania:
                    # pobieramy rwnymi kawakami po rsglobal.SOCK_BUF_OPTIMUM
                    dlugosc_bufora = rsglobal.SOCK_BUF_OPTIMUM
                    while do_pobrania >= dlugosc_bufora:
                        pobrane = self._pobierz_kawalek(dlugosc_bufora)
                        self._juz_odebrane.append(pobrane)
                        do_pobrania -= len(pobrane)

                # pobieramy to co zostao
                while do_pobrania > 0:
                    pobrane = self._pobierz_kawalek(do_pobrania)
                    self._juz_odebrane.append(pobrane)
                    do_pobrania -= len(pobrane)

                # przerabiamy kolejk na napis i czycimy j:
                odebrane = ''.join(self._juz_odebrane)
                del self._juz_odebrane[:]
            else:
                # przerabiamy cz kolejki na napis i czycimy tylko t cz:
                odebrane = ''.join(self._juz_odebrane[:calkowita_dlugosc])
                del self._juz_odebrane[:calkowita_dlugosc]

        finally:
            self._blokada_spr.release()

        # (sprawdzenie w celach debuggingowych)
        assert len(odebrane) == calkowita_dlugosc, ('dugo realnie'
              ' odebranych danych (%s) jest rna od danej (%s)'
              % (len(odebrane), calkowita_dlugosc))

        return odebrane


    # wbrew nazwie (bedacej zaszocia historyczn) zachowanie tej metody
    # istotnie rni si od standardowej socketowej metody recv()
    def recv(self, max_dlugosc=None):
        "Odbieramy napis wraz z info o jego dugosci (wg ustalonego protokou)."

        try:
            # pierwsza_wartosc (<= pierwszy napis majcy dugo ASOCK_DLUGOSC_PIERW):
            # * wg starego sposobu: liczba dodatnia = dugo waciwych danych;
            # * wg nowego sposobu: liczba ujemna = -dugo info dotyczcego dugoci
            #   waciwych danych.
            pierwsza_wartosc = int(self.odbierz_raz(rsglobal.ASOCK_DLUGOSC_PIERW))

            if pierwsza_wartosc < 0:
                # nowym sposobem:
                dlugosc_info_o_dlugosci = (-pierwsza_wartosc)
                if (max_dlugosc is not None) and dlugosc_info_o_dlugosci > rsglobal.MAX_DLUGOSC_INFO_O_DLUGOSCI:
                    raise ValueError('dlugosc_info_o_dlugosci == %d > %d'
                                     % (dlugosc_info_o_dlugosci, rsglobal.MAX_DLUGOSC_INFO_O_DLUGOSCI))
                wlasciwa_dlugosc = int(self.odbierz_raz(dlugosc_info_o_dlugosci))
            else:
                # starym sposobem:
                wlasciwa_dlugosc = pierwsza_wartosc

            if (max_dlugosc is not None) and wlasciwa_dlugosc > max_dlugosc:
                raise ValueError('wlasciwa_dlugosc (%d) > max_dlugosc (%d)'
                                 % (wlasciwa_dlugosc, max_dlugosc))
            return self.odbierz_raz(wlasciwa_dlugosc)

        except Exception, exc:
            rsglobal.ZapiszDoLogow('BD AnakondaSocket.recv(): '
                                    + traceback.format_exc(), 0)
            # [TODO? info u userze do logw (a wic przechwyci to dalej...)]
            raise socket.error('Bd AnakondaSocket.recv() ["%s"]' % exc)


    def wyslij_raz(self, co_wyslac):
        "Wysyamy napis."

        self._blokada_spr.acquire()
        try:
            # (w razie problemu socket.sendall() zgasza wyjtek)
            rezultat = self.socket.sendall(co_wyslac)
        finally:
            self._blokada_spr.release()
        return rezultat


    def wyslij_plik(self, sciezka, kompresja=DOMYSL_KOMPRESJA):
        "Wysyamy zawarto wskazanego pliku, opcjonalnie z kompresj."

        try:
            if kompresja is None:
                # jeli nie okrelono czy kompresowa, kompresujemy dla pocze jest nielokalnych
                if not self.lokal:
                    kompresja = 1
                else:
                    kompresja = 0
            if kompresja:
                wielkosc_pliku = -1  # z kompresj, wic niewiadoma
            else:
                wielkosc_pliku = osp.getsize(sciezka)
            self._blokada_spr.acquire()
            try:
                plik = open(sciezka, 'rb')
                try:
                    self.socket.sendall('%*d' % (rsglobal.MAX_DLUGOSC_INFO_O_DLUGOSCI,
                                                 wielkosc_pliku))
                    if kompresja:
                        wysylacz = self._wysylacz.open()
                        try:
                            plik_wys = gzip.GzipFile('', 'wb', kompresja, wysylacz)
                            try:
                                while True:
                                    co_wyslac = plik.read(rsglobal.SOCK_BUF_WYSYL_PLIK)
                                    if not co_wyslac:
                                        break
                                    plik_wys.write(co_wyslac)
                                plik_wys.flush()
                            finally:
                                plik_wys.close()
                        finally:
                            wysylacz.close()
                    else:
                        while True:
                            co_wyslac = plik.read(rsglobal.SOCK_BUF_WYSYL_PLIK)
                            if not co_wyslac:
                                break
                            self.socket.sendall(co_wyslac)
                finally:
                    plik.close()
            finally:
                self._blokada_spr.release()
        except Exception, exc:
            rsglobal.ZapiszDoLogow('BD AnakondaSocket.wyslij_plik(): '
                                    + traceback.format_exc(), 0)
            # [TODO? info u userze do logw (a wic przechwyci to dalej...)]
            raise socket.error('Bd AnakondaSocket.wyslij_plik() ["%s"]' % exc)


    # wbrew nazwie (bedacej zaszocia historyczn) zachowanie tej metody
    # istotnie rni si od standardowej socketowej metody send()
    def send(self, string):
        "Wysyamy napis wraz z info o jego dugoci (wg ustalonego protokou)."

        try:
            # (napis z informacj o dugoci waciwych danych:
            # uzupeniamy go zerami, by mia *co najmniej* ASOCK_DLUGOSC_PIERW znakw)
            info_o_dlugosci = '%0*i' % (rsglobal.ASOCK_DLUGOSC_PIERW, len(string))

            if len(info_o_dlugosci) > rsglobal.ASOCK_DLUGOSC_PIERW:
                # nowym sposobem:

                # napis pierwsze_info (majcy dugo ASOCK_DLUGOSC_PIERW)
                # zawiera dopenion zerami liczb ujemn == (-1) * dugo
                # napisu z informacj o dugoci waciwych danych
                pierwsze_info = '%0*i' % (rsglobal.ASOCK_DLUGOSC_PIERW,
                                          -len(info_o_dlugosci))

                # (sprawdzenie w celach debuggingowych)
                assert len(pierwsze_info) == rsglobal.ASOCK_DLUGOSC_PIERW, (
                      'Wyglda, jakby dugo pierwszego info, majcego posta "%s"'
                      ' nie zgadzaa si ze sta rsglobal.ASOCK_DLUGOSC_PIERW (%s)'
                      % (pierwsze_info, rsglobal.ASOCK_DLUGOSC_PIERW))

                wyslac_po_kolei = (pierwsze_info, info_o_dlugosci, string)

            else:
                # starym sposobem:
                wyslac_po_kolei = (info_o_dlugosci, string)

            # wysyamy:
            for napis in wyslac_po_kolei:
                self.wyslij_raz(napis)

        except Exception, exc:
            rsglobal.ZapiszDoLogow('BD AnakondaSocket.send(): '
                                    + traceback.format_exc(), 0)
            # [TODO? info u userze do logw (a wic przechwyci to dalej...)]
            raise socket.error('Bd AnakondaSocket.send() ["%s"]' % exc)


    # Metoda sprawdzajca, czy nie zerwano poczenia:
    # (efektem ubocznym moe by odbir minimalnej dugoci danych;
    # zostan one zapamitane do pniejszego uycia)
    def _czy_polaczenie_ok(self):

        try:
            # odbir nieblokujcy:
            oryg_timeout = self.socket.gettimeout()
            self.socket.settimeout(0.0)
            znak = self.socket.recv(1)
            # ^ pusty -> poczenie ju nie funkcjonuje
        except Exception, exc:
            if isinstance(exc, socket.error):
                # (w Pythonie 2.4 socket.error nie ma atrybutu errno)
                exc_errno = exc.args[0]
                if exc_errno in (errno.EPIPE, errno.EBADF):
                    # bd "broken pipe" lub "bad file descriptor"
                    # -> poczenie ju nie funkcjonuje
                    znak = False
                else:
                    # poza tym si nie przejmujemy, bo wyjtek jest typowym
                    # zachowaniem, jeeli akurat nic nie jest przesyane:
                    znak = True
            elif isinstance(exc, SSL.Error):
                if isinstance(exc, (SSL.SysCallError, SSL.ZeroReturnError)):
                    # -> poczenie ju nie funkcjonuje:
                    znak = False
                else:
                    znak = True
            else:
                znak = True
                rsglobal.ZapiszDoLogow('Dziwne: w utworzonym przez %s wtku'
                                       ' sprawdzajcym cigo poczenia,'
                                       ' w metodzie _czy_polaczenie_ok()'
                                       ' przechwycono inny wyjtek ni'
                                       ' instancja socket.error lub'
                                       ' OpenSSL.SSL.Error: ' % self
                                       + traceback.format_exc(), 3)
        else:
            # jeli co zostao odebrane, zachowujemy to do pniejszego uycia
            if znak:
                self._juz_odebrane.append(znak)

        self.socket.settimeout(oryg_timeout)
        #rsglobal.ZapiszDoLogow("znak: " + repr(znak), 5)
        return bool(znak)


    # Metoda dziaajca w osobnym wtku -- budzca si co
    # ODSTEP_SPR_POLACZENIA (lub rzadziej, jeeli akurat dziaaj metody
    # do odbioru/wysyania) i sprawdzajca czy poczenie nie zostao zerwane;
    # jeli zostao, uruchamiana jest podana akcja...
    # (UWAGA: akcja nie bdzie wykonana, jeeli zerwanie poczenia zostanie
    # wykryte gdzie indziej czyli w metodzie do odbioru/wysyania):
    def _czuwaj_czy_nie_zerwano(self, akcja_w_razie_zerwania):

        while self.czy_sprawdzac:
            time.sleep(rsglobal.ODSTEP_SPR_POLACZENIA)
            rsglobal.ZapiszDoLogow('* AnakodaSocket._czuwaj_czy_nie_zerwano():'
                                   ' sprawdzanie poczenia... (socket: %s)'
                                   % self, 5)
            self._blokada_spr.acquire()
            try:
                if not self._czy_polaczenie_ok():
                    rsglobal.ZapiszDoLogow('Wykryto zerwanie poczenia'
                                           ' (socket: %s). Zostanie'
                                           ' wykonana akcja %s'
                                           % (self, akcja_w_razie_zerwania),
                                           4)
                    akcja_w_razie_zerwania()
                    break
            finally:
                try:
                    self._blokada_spr.release()
                except thread.error:
                    pass


    def wlacz_sprawdzanie_polaczenia(self, akcja_w_razie_zerwania):
        "Stwrz wtek reagujcy na zerwanie poczenia."

        # akcja_w_razie_zerwania - bezargumentowa funkcja
        if not self.czy_sprawdzac:
            self.czy_sprawdzac = True
            thread.start_new_thread(self._czuwaj_czy_nie_zerwano,
                                    (akcja_w_razie_zerwania,))
        else:
            rsglobal.ZapiszDoLogow('Podjto jake niesuszn prb wczenia'
                                   ' ju wczonego sprawdzania poczenia'
                                   ' (socket: %s)' % self, 3)


    def accept(self):
        "Akceptujemy poczenie i inicjujemy nim nowy obiekt AnakondaSocket."

        conn, addr = self.socket.accept()
        return AnakondaSocket(socket_obj=conn), addr


    def close(self):
        self.czy_sprawdzac = False  # koczymy ew. wtek sprawdzajcy poczenie
        return self.socket.close()


    def shutdown(self, how=socket.SHUT_RDWR):
        self.czy_sprawdzac = False  # koczymy ew. wtek sprawdzajcy poczenie
        return self.socket.shutdown(how)


    def ustaw_keepalive(self):
        self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)  # wcz
        self.socket.setsockopt(socket.SOL_TCP, socket.TCP_KEEPCNT, 9)      # ile prb sprawdz.
        self.socket.setsockopt(socket.SOL_TCP, socket.TCP_KEEPIDLE, 80)    # spr. po ilu s
        self.socket.setsockopt(socket.SOL_TCP, socket.TCP_KEEPINTVL, 75)   # spr. co ile sek
